/* glXml.cc - 3-d visualizer for the xml interface
*/


#include <stdio.h>
#include <GL/gl.h>
#include <GL/glut.h>

#include "physics.h"
#include "rs-xml.h"

#define DEFAULT_RADIUS ( 1.00 )

void init(char* File);
void display(void);
void reshape(int w, int h);
void keyboard(unsigned char, int, int);
void update(void);

// Global variables
universe* Uni = NULL;
scalar* Vec_data = NULL;

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	
	// arg check
	if (argc < 2) {
		printf("Usage: %s <input.xml>\n", argv[0]);
		exit(1);
	}
	
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutCreateWindow(argv[0]);
	init(argv[1]);
	
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutIdleFunc(update);
	
	glutMainLoop();
	
	return 0;
	
}

void init(char* File)
{
	fprintf(stderr, "init()\n");
	universe_xml_interface Input(File);
	
	Uni = Input.output();
	
	if (Uni == NULL) {
		fprintf(stderr, "Error loading %s!!\n", File);
	}
	
	Uni->calculateDamping();
	
	// Find out where vector return data is kept
	Vec_data = Uni->dampingCoeff().array();
	fprintf(stderr, "dampingCoeff: %g %g %g\n", Vec_data[0], Vec_data[1],
	        Vec_data[2]);
	
	// Now set up openGL
	GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat mat_shininess[] = {50.0};
	GLfloat light_position[] = {1.0, -0.3, 0.5, 0.0};

	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	
	// Lighting
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);

	// make everything blue
	glColor3f(0.0, 0.0, 1.0);
	multi_vector* S = Uni->position();
	for (int i = 0; S[i].id != 0; i++) {
		S[i].data.array();
		fprintf(stderr, "obj %d: (%g, %g, %g)\n", S[i].id,
			Vec_data[0], Vec_data[1], Vec_data[2]);
   	}

	fprintf(stderr, "init() done.\n");
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	multi_vector* S = Uni->position();

	for (int i = 0; S[i].id != 0; i++) {
		scalar r = Uni->radius(S[i].id);
		if (r == 0.0) {
			r = DEFAULT_RADIUS;
		}
		S[i].data.array();
		glPushMatrix();
		glTranslatef(Vec_data[0], Vec_data[1], Vec_data[2]);
		glutSolidSphere(r, 20, 16);
		glPopMatrix();
	}
	
	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0, 50.0, 0.0, 50.0, -100.0, 100.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}

void keyboard (unsigned char key, int x, int y)
{
	fprintf(stderr, "keyboard()\n");
	switch (key) {
		case 27:
			// ESC
			exit(0);
			break;
		case 'p':
			Uni->togglePause();
			break;
		default:
			break;	
	}
	fprintf(stderr, "keyboard() done.\n");
}

// RockSpace logic
void update(void)
{
	static scalar T = Uni->time();
	static int i = 0;

	if (Uni->time() - T > 5.0) {
		scalar delta_T = Uni->time() - T;
		printf("%d frames in %.2f seconds.  %.2f fps.\n",
		        i, delta_T, i / delta_T);
		T = Uni->time();
	}
		
	// Want to update all these interactions.
	// We'll assume gravity doesn't need to be checked.
	Uni->calculateDamping();
	Uni->calculateSpring();
	Uni->calculateCollision();

	
	i++;
	glutPostRedisplay();
		
}
