// springs.cc - test out hooke's law interaction

// elasticity of materials leads to restoring force proportional to
// displacement.

#include <stdio.h>
#include "physics.h"

int main(void)
{
	object Post("post", 12000.0, ToVector(15.0, 0.0, 0.0)),
	       Ball("ball", 15.0, ToVector(15.0, 0.0, 0.0)),
	       Earth("earth", 5.93e24, ToVector(-6.37e6, 0.0 ,0.0));
	
	scalar X = Post.position().x(), Y = Ball.position().x();
	
	hookes_law* Spring = new hookes_law(&Post, &Ball, 19.6);
	
	
	printf("%s: (%g)\t%s: (%g)\n", Post.name(), X, Ball.name(), Y);
	
	gravitation Gravity(&Earth, &Ball);
	vector a = Ball.acceleration();
	printf("a = (%g, %g, %g)\n", a.x(), a.y(), a.z());
	
	int i = 0;
	scalar T_zero = Earth.time();
	
	while (Y > 0.1512) {
		X = Post.position().x();
		Y = Ball.position().x();
//		printf("%s: %g\t%s: %g\r", Post.name(), X,
//		       Ball.name(), Y);
		Post.setPosition(ToVector(15.0, 0.0, 0.0));
		Spring->calculate();
		i++;
	}

	scalar T_final = Earth.time();
		
	printf("\n");
	printf("%d loops in %g seconds: %g loops/sec\n", i,
	       T_final - T_zero, i / (T_final - T_zero));
	
/*	// Cut the spring at the bottom of the drop.
	delete Spring;
	while (Y > -50.0) {
		X = Post.position().x();
		Y = Ball.position().x();
		printf("%s: %g\t%s: %g\r", Post.name(), X,
		       Ball.name(), Y);
	}
        printf("\n");
*/	
	return 0;
	
}