/* rs-xml.h -- XML I/O interface for the RockSpace physics engine.
*/

#ifndef ROCKSPACE_XML_H
#define ROCKSPACE_XML_H

#include "physics.h"
#include <libxml/parser.h>
#include <libxml/tree.h>

typedef struct _id_list
{
	object_id RS_id;
	int       XML_id;
	struct _id_list* next;
} id_list;


class universe_xml_interface
{
	private:
	xmlDoc* _doc;
	xmlNode* _root;
	id_list* _id_table;
	universe* _uni;
	
	// Remove any pre-existing data
	void _clear_data(void);
	
	public:
	universe_xml_interface(void);
	
	// Load information from a given XML file
	universe_xml_interface(char* Filename);
	
	// Load information from an existing universe
	universe_xml_interface(universe* Uni);
	~universe_xml_interface(void);
	
	// I/O Methods:
	// Returns universe stored in memory.  The universe is created when
	// the XML file is read
	inline universe* output(void) const {return _uni;}
	
	// Create an in-memory XML representation of an existing universe
	// This will re-create the XML tree.
	bool input(universe* Uni);
	
	// Load & parse XML file
	bool load(char* Filename);
	
	// Save XML representation to file.  No effect if none is loaded.
	inline int save(char* Filename) const
	   {return xmlSaveFile(Filename, _doc);}
	
	// Manipulation methods:
	// These are intended to allow users to extend the functionality of
	// the XML format used by the engine.
	
	// Return root of DOM tree describing universe
	inline xmlNode* root(void) const {return _root;}
	
	// Return document pointer
	inline xmlDoc* document(void) const {return _doc;}
	
	// These will allow us to cross-identify objects in XML and RockSpace
	object_id rsId(int Xml_Id) const;
	int xmlId(object_id Rs_Id) const;
	
	// Add a blank id container to the end of the list.
	friend id_list* Add_Id_Entry(void);
};

// Some useful external functions
vector Extract_Vector(xmlNode* Node);
char* Extract_Attr(xmlNode* Node, char* Name);


#endif
