// Vector class definition

#ifndef VECTOR_H
#define VECTOR_H
// Maximum length of the vector.  Smaller means less memory usage, duh.
#define VECTOR_MAX 3

// A useful constant
// #define ZERO_VECTOR ToVector(0.0, 0.0, 0.0)
// see the end...

typedef double scalar;

class vector
{

	static scalar _result[VECTOR_MAX];
	
	int length;
	scalar data[VECTOR_MAX];

	public:

	// Display the vector to stdout.
	void disp(void);
	
	// Convinient operators -- making C++ more like MATLAB
	friend vector operator +(const vector& a, const vector& b);
	friend vector operator -(const vector& a, const vector& b);
	friend vector operator *(const vector& a, const scalar& b);
	friend vector operator *(const scalar& b, const vector& a);
	friend vector operator /(const vector& a, const scalar& b);
	friend vector operator -(const vector& a);
	vector operator =(const scalar &s);
	
	friend bool operator ==(const vector& a, const vector& b);
	friend bool operator !=(const vector& a, const vector& b);

	// Initialize the vector with so many (Count) double-prec. #s
	void assign(const int Count, ...);

	//  assign's little (real-world) bro
	void assign3d(const scalar A, const scalar B, const scalar C);

	// Vector dot product
	scalar dot(const vector A) const;

	// Vector cross product
	vector cross(const vector A) const;

	// Magnitude - returns scalar
	scalar mag(void) const;

	// Direction - returns unit vector in same direction.
	vector dir(void) const;

	// Returning individual components
	scalar access(int Index) const;
	scalar x(void) const;
	scalar y(void) const;
	scalar z(void) const;
	
	// Return as a block array
	scalar* array(void) const;

	// default (dumb) constructor
	vector(void);

	// constructor for 3-D vectors
	vector(const scalar A, const scalar B, const scalar C);


};

// Useful external vector functions.
// duh.  i coulda used the constructor all this time.  think everything's
// been changed.
vector ToVector(const scalar A, const scalar B, const scalar C);

// global variable to be used as the zero vector.
extern const vector ZERO_VECTOR;

#endif