/* io.cc - Methods and functions used to create, display, and access the
 * contents of vectors.
 *
 *  By Rocky Scaletta
 *  June 10, 1999
 *
 *  This software is being offered in the hope that it will prove useful, but
 *  with NO WARRANTY.  You are free to make any changes you want, but any future
 *  copies must retain this notice, yada yada.
 *
 * Check out the GPL version 2.
 */

#include <stdio.h>
#include <stdarg.h>
#include "vector.h"

/*******************************************************************************
 * First, some functions and methods used to create vectors and fill them with *
 * values.                                                                     *
 *******************************************************************************/

/* a constructor.  what a concept!  should have added this ages ago.
   specifically handles only 3-D vectors.
   No dynamically allocated memory, therefore no destructor.
*/
vector::vector(const scalar A, const scalar B, const scalar C)
	: length(3)
{
	data[0] = A;
	data[1] = B;
	data[2] = C;
}

// general dumb constructor
vector::vector(void) : length(0)
{
}

// Assign an arbitrary number of scalar values to a vector.
// There should be 'Count' scalars following Count itself.
void vector::assign(const int Count, ...)
{
	int I;
	va_list vPtr;

	if (Count <= VECTOR_MAX)
	{
		va_start(vPtr, Count);
	
		for (I = 0; I < Count; I++)
			data[I] = va_arg(vPtr, scalar);
		length = Count;

		va_end(vPtr);
	}
	else
	{
		length = 0;
	}
	
	return;
}

// Here's a more optimized assignment method good for vectors with 3 elements.
void vector::assign3d(const scalar A, const scalar B, const scalar C)
{
	length = 3;
	data[0] = A;
	data[1] = B;
	data[2] = C;

	return;
}

// Here's a function that _returns_ a vector
// I use it when i want a vector parameter but i don't want an extra variable.
vector ToVector(const scalar A, const scalar B, const scalar C)
{
	vector New;

	New.assign3d(A, B, C);

	return New;
}

/*****************************************************************************
 * Now methods for accessing the contents of a vector.  Mostly we want them  *
 * as scalar values, but it's useful to see them printed on stdout sometimes,*
 * too.                                                                      *
 *****************************************************************************/

// Display the contents of the vector to stdout.
void vector::disp(void)
{
	int I;
	for (I = 0; I < length; I++)
		printf("%g  \t", data[I]);
	printf("\n");
}

// Here we return the element specified by 'Index.' Vector is zero-indexed.
// Return zero on error, i guess...  maybe i need to implement
// some exception handling?
scalar vector::access(int Index) const
{
	if (Index >= length)
	{
		fprintf(stderr,"Tried to access element out of range!!\n");
		return 0;
	}
	else
	{
		return data[Index];
	}
}

// Each of these methods returns the specified coordinate ... useful when we're
// dealing with 3-d vectors.
scalar vector::x(void) const
{
	return data[0];
}

scalar vector::y(void) const
{
	return data[1];
}

scalar vector::z(void) const
{
	return data[2];
}

// It might be nice to have an old-school array to work with.
// I'll put the values into a static array.  This way, there won't be any
// memory leaks.  As a result, every call to array() resets the array!
// I guess if the user really wants a copy of the array, (s)he can deal with
// it.
scalar vector::_result[VECTOR_MAX];

scalar* vector::array(void) const
{

	for (int i = 0; i < length; i++) {
		_result[i] = data[i];
	}
	
	return _result;
}