/* operators.cc - Vector math operators
 *
 *  By Rocky Scaletta
 *  June 10, 1999
 *
 *  This software is being offered in the hope that it will prove useful, but
 *  with NO WARRANTY.  You are free to make any changes you want, but any future
 *  copies must retain this notice, yada yada.
 *
 * Check out the GPL version 2.
 */

#include "vector.h"

// If we run into trouble (bad dimensions) just set the length of the vector to
// zero.  They'll figure it out, yo.

vector operator +(const vector& a, const vector& b)
{
	int I;
	vector c;

	if (a.length == b.length) {
		c.length = a.length;
		for (I = 0; I < a.length; I++)	{
			c.data[I] = a.data[I] + b.data[I];
		}
	}

	return c;
}

vector operator *(const vector& a, const scalar& b)
{
	vector c;

	c.length = a.length;
	for (int I = 0; I < a.length; I++)
	{
		c.data[I] = a.data[I] * b;
	}
	return c;
}

vector operator *(const scalar& b, const vector& a)
{
	vector c;

	c.length = a.length;
	for (int I = 0; I < a.length; I++)
	{
		c.data[I] = a.data[I] * b;
	}
	return c;
}

// Don't divide by zero, please.
vector operator /(const vector& a, const scalar& b)
{
	vector c;

	c.length = a.length;
	for (int I = 0; I < a.length; I++)
	{
		c.data[I] = a.data[I] / b;
	}
	return c;
}

// Subtraction
vector operator -(const vector& a, const vector& b)
{
	return vector(a + (-1 * b));
}

// Negation, opposite, whatever
vector operator -(const vector& a)
{
	return (-1 * a);
}

// only creates a 3-vector.  hmmm
vector vector::operator =(const scalar& s)
{
	return vector(s, s, s);
}

// Equality/inequality
bool operator ==(const vector& a, const vector& b)
{
	if (a.length != b.length) {
		return false;
	}

	for (int i = 0; i < a.length; i++) {
		if (a.data[i] != b.data[i]) {
			return false;
		}
	}
	
	return true;
}

bool operator !=(const vector& a, const vector& b)
{
	return !(a == b);
}