/* properties.cc - All the little things that make vectors grand, like magnitude,
 * direction, etc.
 *
 *  By Rocky Scaletta
 *  June 10, 1999
 *
 *  This software is being offered in the hope that it will prove useful, but
 *  with NO WARRANTY.  You are free to make any changes you want, but any future
 *  copies must retain this notice, yada yada.
 *
 * Check out the GPL version 2.
 */


#include <math.h>
#include "vector.h"

// I guess a property of the vector class would be that it can be zero...
const vector ZERO_VECTOR(0.0, 0.0, 0.0);

// Dot (scalar) product.  Remember, dot product doesn't care what order its
// arguments are in.
scalar vector::dot(const vector A) const
{
	int I;
	scalar Sum = 0;

	if (length == A.length)
	{
		for (I = 0; I < length; I++)
		{
			Sum += (data[I] * A.data[I]);
		}
	}

	return Sum;
}

//  Cross product for 3-d vectors, using 3x3 matrices
//  probably should make a n-d version, but that's a lot more work.
//  When you say "vector_1.cross(vector_2)" returns vector_1 x vector_2.
//  This is important because with cross products, order _does_ matter.
vector vector::cross(const vector A) const
{
	vector C;

	if (length != 3 || A.length != 3)
	{
		C.length = 0;
	}
	else
	{
		C.length = 3;

		C.data[0] = (data[1] * A.data[2] - data[2] * A.data[1]);
		C.data[1] = (data[0] * A.data[2] - data[2] * A.data[0]);
		C.data[2] = (data[0] * A.data[1] - data[1] * A.data[0]);
	}

	return C;
}

// Return magnitude of the vector
scalar vector::mag(void) const
{
	scalar Sum = 0;
	int I;

	for (I = 0; I < length; I++)
	{
		Sum += data[I] * data[I];
	}

	return sqrt(Sum);
}

// Returns direction of vector, which is a unit vector in the same direction as
// the vector in question.
// You can accomplish the same thing by running "C = A / A.mag()"
// Not sure which one is more efficiently implemented...
vector vector::dir(void) const
{
	vector C;
	scalar Mag = 0.0;
	int I;

	C.length = length;

	for (I = 0; I < length; I++)
	{
		Mag += data[I] * data[I];
	}

	Mag = sqrt(Mag);

	for (I = 0; I < length; I++)
	{
		C.data[I] = data[I] / Mag;
	}

	return C;
}

