/* object.cc  -- deals with basic object manipulation
                 construction, population, destruction, etc.

   for RockSpace v2.0
   (C) 2000 Rocky Scaletta.
   see file COPYING in base folder for license details.
*/

#include "object.h"

// For NULL
#include <stddef.h>

// For dealing with names of objects.

// Limit the force list to this many entries.  Easier than dealing with
// dynamic creation.
// Note that when you add gravity, this is an effective limit on objects in
// the universe.
#define MAX_FORCES 1024

// Change this value to change the apparant rate of time
#define DEFAULT_TIME_X (1.00)


// Initialize and blank every thing.
object::object(void)
{
	_name = NULL;
	_position = _velocity = ZERO_VECTOR;
	_mass = 0.0;
	
	_init_time = _now();
	_time_multiplier = DEFAULT_TIME_X;
	
	_force_list = NULL;
	_resultant = ZERO_VECTOR;
	_paused = false;
}

// Default arguments are nifty!
object::object(char *Name, scalar M, vector S, vector V)
{
	_name = new char[strlen(Name) + 1];
	if (_name != NULL) {
		strcpy(_name, Name);
	}
	
	_mass = M;
	_position = S;
	_velocity = V;
	
	_init_time = _now();
	_time_multiplier = DEFAULT_TIME_X;
	
	_force_list = NULL;
	_resultant = ZERO_VECTOR;
	_paused = false;
	
}

object::~object(void)
{
	delete [] _name;
	_remove_all_forces();
}

// Should we created an anonymous object to return this?  Don't know if I
// like handing out the raw pointer, yo.
char* object::name(void) const
{
	return _name;
}

// Since we hand out said pointer, be careful here.
bool object::setName(char* New_Name)
{
	// first, clear old name.
	delete [] _name;
	_name = NULL;
	
	_name = new char[strlen(New_Name) + 1];
	if (_name != NULL) {
		strcpy(_name, New_Name);
		return true;
	}

	return false;
}