/* collisions.cc -- (in)elastic collisions
        The one physical law that has lasted through relativity and all the
   other discoveries of the 20th century has been the conservation of
   momentum.
        Ideally, we'd use this law (p = p') along with the amount of kinetic
   energy conserved to calculate new velocities for each object.  In
   practice, these equations in their general forms are very complex.  For
   now, I'm just going to fudge a bit.
        Along with "at-will" collisions, it would be relatively easy to add
   automatic collision detection with another interaction, along the lines
   of gravity.  The design of RockSpace lends itself to spherical "bounding
   boxes;" I think I'll leave more complex schemes to users of the engine.
*/


#include "physics.h"
#include <stddef.h>
#include <math.h>
#include <assert.h>

#define PI ( 3.14159265358979 )

#define ZERO_THRESHOLD ( 1e-4 )


// debug
#include <stdio.h>




// Check entire universe for colliding objects.
bool universe::calculateCollision(void)
{
	if (_object_list == NULL) {
		return false;
	}

	_on_every_object(_object_list, Refresh_Collision);
	return true;
}

void Refresh_Collision(point_mass* Object)
{
	if (Object == NULL)
		return;

	interaction_node* Here = Object->_object_borders;
	while (Here != NULL) {
		Here->data->calculate();
		Here = Here->next;
	}
	return;

}

// Perhaps more useful to return true if a collision occurs?  Would require
// calculate() method to not return void...
bool universe::calculateCollision(object_id Id)
{
	point_mass* Target = _get_by_id(Id);

	if (Target == NULL) {
		return false;
	}

	// Search through object's list of borders, performing a check
	// each time.
	interaction_node* Here = Target->_object_borders;
	while (Here != NULL) {
		Here->data->calculate();
		Here = Here->next;
	}

	return true;
}