/* damping.cc - Dealing with friction
        Without friction, we have nothing.  Indeed.
   It's calculated as a quadratic equation with respect to velocity:
   F = -(a*v^2 + b*v + c)
   The negative sign tells us this force will always oppose motion.
*/

#include "physics.h"

// for NULL
#include <stddef.h>

void Update_Friction(point_mass* Here);

vector universe::dampingCoeff(void)
{
	return ToVector(_damping_coeff[0], _damping_coeff[1],
	                _damping_coeff[2]);
}

vector universe::dampingCoeff(object_id Id)
{
	point_mass* Target = _get_by_id(Id);
	if (Target == NULL) {
		return dampingCoeff();
	} else {
		return Target->_friction->coeff();
	}
}

bool universe::setDampingCoeff(vector K)
{
	scalar* Result = K.array();
	for (int i = 0; i < 3; i++) {
		_damping_coeff[i] = Result[i];
	}
	
	return true;
}

bool universe::setDampingCoeff(object_id Id, vector K)
{
	point_mass* Target = _get_by_id(Id);
	if (Target == NULL) {
		return false;
	} else {
		return Target->_friction->setCoeff(K);
	}
}

bool universe::calculateDamping(void)
{
	_on_every_object(_object_list, Update_Friction);
	return true;
}

void Update_Friction(point_mass* Here)
{
	Here->_friction->calculate();
}

bool universe::calculateDamping(object_id Id)
{
	point_mass* Target = _get_by_id(Id);
	if (Target == NULL) {
		return false;
	} else {
		Target->_friction->calculate();
		return true;
	}
}