/* input.cc - change attributes of objects directly.
        These methods provide a counterpoint to output.cc.  It is simpler to
   change an object's attributes directly with these methods, although
   definitely less realistic and automated than using forces.  I would also
   recommend against using "setAcceleration()" because it has a habit of
   clobbering all forces acting on an object in favor of its own.  As a
   result, gravity and springs will simply stop working on the object.
*/

#include "physics.h"
#include <stddef.h>

bool universe::setPosition(object_id Id, vector S)
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return false;
	} else {
		return Obj->setPosition(S);
	}
}

bool universe::setVelocity(object_id Id, vector V)
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return false;
	} else {
		return Obj->setVelocity(V);
	}
}

bool universe::setAcceleration(object_id Id, vector A)
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return false;
	} else {
		return Obj->setAcceleration(A);
	}
}

bool universe::setMass(object_id Id, scalar M)
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return false;
	} else {
		return Obj->setMass(M);
	}
}