/* object_control.cc - simple, direct control over an individual object.
        The idea here is that the universe will "bud" control objects.
   These objects provide an interface to apply force to an object: to push
   it around.  These forces are of a transient nature; when the control box
   is destroyed, the force is removed.
*/

#include "physics.h"
// For NULL
#include <stddef.h>

// First define the members of the object_control class

object_control::object_control(void)
{
	_obj = NULL;
	_id = 0;
	_force = 0;
}

object_control::object_control(object* Obj)
{
	if (Obj != NULL) {
		_obj = Obj;
		_id = _obj->id();
		_force = _obj->applyForce(ZERO_VECTOR);
	}
}

object_control::~object_control(void)
{
	if (_obj != NULL) {
		_obj->removeForce(_id);
	}
}

// Take control of an object, shift focus.
bool object_control::control(object* Obj)
{
	if (Obj != NULL) {
		if (_obj != NULL) {
			_obj->removeForce(_force);
		}
		_obj = Obj;
		_id = _obj->id();
		_force = _obj->applyForce(ZERO_VECTOR);
		return true;
	} else {
		return false;
	}
}

bool object_control::setForce(vector F)
{
	if (_obj != NULL) {
		return (_obj->setForce(_force, F));
	} else {
		return false;
	}
}

vector object_control::forceIs(void) const
{
	if (_obj != NULL) {
		return _obj->getForce(_force);
	} else {
		return ZERO_VECTOR;
	}
}

// Now here's the universe interface that will sprout control structures.
object_control universe::control(object_id Id)
{
	point_mass* Obj = _get_by_id(Id);
	return object_control(Obj);
}