/* output.cc - Returning data from the universe.
        Perhaps the most important part of the physics engine.  Without it,
   the whole thing is worthless; there would be no way to use the
   information.
        In general, I'll support two methods for retrieving data.  First, if
   an ID # is specified, a single value will be retrieved.  Second, if no ID
   is given, a null-terminated array of "multi_vector"s will be returned.
   The multi-vector structure contains corresponding object id's and
   vectors.  There is an analagous "multi_scalar" type as well.
*/

#include "physics.h"
#include <stddef.h>

int Count_Objects(point_mass* Here);

// First off, the singular return values.

vector universe::position(object_id Id) const
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return ZERO_VECTOR;
	} else {
		return Obj->position();
	}
}

vector universe::velocity(object_id Id) const
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return ZERO_VECTOR;
	} else {
		return Obj->velocity();
	}

}

vector universe::acceleration(object_id Id) const
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return ZERO_VECTOR;
	} else {
		return Obj->acceleration();
	}
}

scalar universe::mass(object_id Id) const
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj == NULL) {
		return 0.0;
	} else {
		return Obj->mass();
	}

}

// Now the multi-value returns.
multi_vector* universe::position(void) const
{
	int Num_Objs = Count_Objects(_object_list);
	multi_vector* Result = new multi_vector[Num_Objs + 1];
        point_mass* Here = _object_list;
	int i = 0;
		
	for (; i < Num_Objs; i++) {
		Result[i].data = Here->position();
		Result[i].id = Here->_id;
		Here = Here->_next;
	}

	// NULL-terminating the end.
	// Basically just setting everything to zero.
	Result[i].data = ZERO_VECTOR;
	Result[i].id = 0;
	
	return Result;	
}

multi_vector* universe::velocity(void) const
{
	int Num_Objs = Count_Objects(_object_list);
	multi_vector* Result = new multi_vector[Num_Objs + 1];
        point_mass* Here = _object_list;
	int i = 0;
		
	for (; i < Num_Objs; i++) {
		Result[i].data = Here->velocity();
		Result[i].id = Here->_id;
		Here = Here->_next;
	}

	// NULL-terminating the end.
	// Basically just setting everything to zero.
	Result[i].data = ZERO_VECTOR;
	Result[i].id = 0;
	
	return Result;	
}

multi_vector* universe::acceleration(void) const
{
	int Num_Objs = Count_Objects(_object_list);
	multi_vector* Result = new multi_vector[Num_Objs + 1];
        point_mass* Here = _object_list;
	int i = 0;
		
	for (; i < Num_Objs; i++) {
		Result[i].data = Here->acceleration();
		Result[i].id = Here->_id;
		Here = Here->_next;
	}

	// NULL-terminating the end.
	// Basically just setting everything to zero.
	Result[i].data = ZERO_VECTOR;
	Result[i].id = 0;
	
	return Result;	
}

multi_scalar* universe::mass(void) const
{
	int Num_Objs = Count_Objects(_object_list);
	multi_scalar* Result = new multi_scalar[Num_Objs + 1];
        point_mass* Here = _object_list;
	int i = 0;
		
	for (; i < Num_Objs; i++) {
		Result[i].data = Here->mass();
		Result[i].id = Here->_id;
		Here = Here->_next;
	}

	// NULL-terminating the end.
	// Basically just setting everything to zero.
	Result[i].data = 0.0;
	Result[i].id = 0;
	
	return Result;	
}

int Count_Objects(point_mass* Here)
{
	static int i = 0;
	
	if (Here == NULL) {
		int Temp = i;
		i = 0;
		return Temp;
	} else {
		i++;
		return Count_Objects(Here->next());
	}
}