/* point_mass.cc - Methods for keeping track of a list of objects.
        Implement objects as a linked list.

*/

#include "physics.h"
#include <stddef.h>

point_mass::point_mass(void)
{
	_id = 0;
	_gravity = NULL;
	_friction = NULL;
	_prev = _next = NULL;
	_radius = 0.0;
	_object_borders = NULL;
	_errata = NULL;
}

point_mass::point_mass(char *Name, scalar Mass,
                       vector Position, vector Velocity, scalar r)
   : object(Name, Mass, Position, Velocity)
{
	_id = 0;
	_gravity = NULL;
	_friction = NULL;
	_prev = _next = NULL;
	_radius = r;
	_object_borders = NULL;
	_errata = NULL;
}

// Little recursive helper.
void Delete_Nodes(interaction_node* Here);

point_mass::~point_mass(void)
{
	_id = 0;
	if ((_next != NULL) && (_prev != NULL)) {
		point_mass* Temp = _next;
		_next->_prev = _prev;
		_prev->_next = Temp;
	} else if ((_next == NULL) && (_prev != NULL)) {
		_prev->_next = NULL;
	} else if ((_next != NULL) && (_prev == NULL)) {
		_next->_prev = NULL;
	}
	
	_prev = _next = NULL;
	Delete_Nodes(_gravity);
	Delete_Nodes(_object_borders);
	_gravity = NULL;
	
	delete _friction;
	_friction = NULL;
	
}

void Delete_Nodes(interaction_node* Here)
{
	if (Here == NULL) {
		return;
	} else {
		Delete_Nodes(Here->next);
		delete Here->data;
		delete Here;
	}
}

point_mass* point_mass::next() const
{
	return _next;
}

bool point_mass::setNext(point_mass* Node)
{
	_next = Node;
	return true;
}

point_mass_id point_mass::id(void) const
{
	return _id;
}	

bool point_mass::setId(point_mass_id Id)
{
	_id = Id;
	return true;
}

scalar point_mass::radius(void) const
{
	return _radius;
}

scalar universe::radius(object_id Id) const
{
	point_mass* Obj = _get_by_id(Id);
	if (Obj != NULL) {
		return Obj->_radius;
	} else {
		return 0.0;
	}
}


// The userData pointer is not controlled by the object, just stored.
// Any memory (de)allocation should be handled by the client program.
void* point_mass::userData(void) const
{
	return _errata;
}

bool point_mass::setUserData(void* Data_Ptr)
{
	_errata = Data_Ptr;
	return true;
}