/* universe.cc - general universe maintanence tools.
        The universe is the glue that binds my world together.  It knows the
   physical laws that govern the theoretical objects, it populates itself,
   it causes objects to interact.
*/

#include "physics.h"
#include <stddef.h>

void Delete_Spring(interaction_node* Here);
bool Match_Id(point_mass* Target, void* Arg);
void Delete_Object(point_mass* Here);

universe::universe(void)
{
	_object_list = NULL;
	_spring_list = NULL;
	_time_x = DEFAULT_TIME_X;
	_paused = false;
	_initial_t = _now();
	for (int i = 0; i < 3; i++)
		_damping_coeff[i] = 0.0;
}

universe::~universe(void)
{
	_on_every_object(_object_list, Delete_Object);
	_on_every_object(_spring_list, Delete_Spring);
}

void universe::_on_every_object(point_mass* Here, void (*fn)(point_mass*))
{
	if (Here == NULL) {
		return;
	} else {
		_on_every_object(Here->next(), fn);
		fn(Here);
	}
}

void universe::_on_every_object(interaction_node* Here,
                                void (*fn)(interaction_node*))
{
	if (Here == NULL) {
		return;
	} else {
		_on_every_object(Here->next, fn);
		fn(Here);
	}
}

void Delete_Object(point_mass* Here)
{
	delete Here;
}

void Delete_Spring(interaction_node* Here)
{
	delete Here->data;
	delete Here;
}

// Useful little guy.
point_mass* universe::_get_by_id(point_mass_id Id) const
{
	return _search(_object_list, &Id, Match_Id);
}

// searches through object list until it hits the end or until the
// comparison function returns true.
point_mass* universe::_search(point_mass* Here, void* Arg,
                    bool (*fn)(point_mass*, void*)) const
{
	if ((Here == NULL) || (fn(Here, Arg))) {
		return Here;
	} else {
		return _search(Here->_next, Arg, fn);
	}
}

// Need to cast the generic void argument back to a usable id, and then
// dereference.
bool Match_Id(point_mass* Target, void* Arg)
{
	if (Target->id() == *((point_mass_id*) Arg))
		return true;
	else
		return false;
}
