/* glTest.cc -- try out an openGL frontend.
        I haven't done any GL stuff for months, and I gave Nick's book back.
   So, this is really just a "proof of concept," as it were.
*/

#include <stdio.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include "physics.h"

#define MASS_MULTIPLIER ( 10.0 )
#define SQRT_2 ( 1.414213562 )
#define nan ( 1 / 0 )

void init(void);
void display(void);
void reshape(int, int);
void keyboard(unsigned char, int, int);
void update(void);


// Global vars for keeping track of objects.
universe Everything;
object_id Ball = 0, Ball_2 = 0;
scalar* V = NULL;
scalar Radii[2];
scalar T = 0.0;

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutCreateWindow("Testing RockSpace");
	init();
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutIdleFunc(update);
	
	glutMainLoop();
	
	return 0;
}

void init(void)
{
	// An object
	Ball = Everything.add(10.0, vector(0.0, 50.0, 0.0),
	                      10.0 * vector(0.707106781186548, -0.707106781186548, 0.0),
	                      1.0);
	Ball_2 = Everything.add(20.0, vector(0.0, 0.0, 0.0),
	                      10.0 * vector(0.707106781186548, 0.707106781186548, 0.0),
	                      1.0);
	// Set up repository for reading position of the object
	V = Everything.position(Ball).array();

	Radii[0] = Everything.mass(Ball) / MASS_MULTIPLIER;
	Radii[1] = Everything.mass(Ball_2) / MASS_MULTIPLIER;

	Everything.calculateDamping();

	GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat mat_shininess[] = {50.0};
	GLfloat light_position[] = {1.0, -0.3, 0.5, 0.0};
	
	// Now set up GL the way we like it.
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
//	glMatrixMode(GL_PROJECTION);
//	glLoadIdentity();
//	glOrtho(0.0, 1.0, 0.0, 50.0, -1.0, 1.0);
//	glMatrixMode(GL_MODELVIEW);

	// Lighting
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);

	glColor3f(0.0, 0.0, 1.0);

	T = Everything.time();
}

void display(void)
{

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Ball
	Everything.position(Ball).array();

	glPushMatrix();
	glColor3f(0.0, 0.0, 1.0);
	glTranslatef(V[0], V[1], V[2]);
	glutSolidSphere(Radii[0], 20, 16);
	glPopMatrix();

	Everything.position(Ball_2).array();
	glPushMatrix();
	glColor3f(0.0, 1.0, 0.0);
	glTranslatef(V[0], V[1], V[2]);
	glutSolidSphere(Radii[1], 20, 16);
	glPopMatrix();


	glutSwapBuffers();

}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0, 50.0, 0.0, 50.0, -100.0, 100.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void keyboard (unsigned char key, int x, int y)
{
	switch (key) {
		case 27:
			// ESC
			exit(0);
			break;
		case 'p':
			Everything.togglePause();
			break;
		default:
			break;
	}
}

// Rockspace logic
void update(void)
{
	static int i = 0;

	// Basically, we should just force a recalculation of the position
	// of the object.  If we call array(), it goes to the vector buffer.
        Everything.calculateDamping();
        Everything.calculateCollision();

	i++;

	if (i > 1000) {
		scalar delta_t = Everything.time() - T;
		printf("%d frames in %.2f seconds: %.2f fps.\n",
		       i, delta_t, i / delta_t);
		i = 0;
		T = Everything.time();
	}
	
	glutPostRedisplay();
}